(***********************************************************************************
 *
 *  Application SRTFS (Software Reuse Through Formal Specifications) 
 *
 *  File: data_ctg.sml
 *
 *  Description: data structures used in the representations of
 *               categorical specifications
 * 
 ***********************************************************************************)

structure Catg =
    struct
	
	structure Spec=Spec;
	
	(* identifier *)
	type id = Spec.id;
	   
	(* type of variable *)
	datatype tvar =    
            in_ of id | 
            out of id; 
	    
        (* type of predicate *)
	datatype predicate =
	    pred of id   (* normal predicate *)
	  | comp of tvar;  (* equality predicate *)


	(* type of assertion *)
	datatype assertion = 
	    prop of predicate*id (* proposition *)
	  | exp of predicate  (* predicate expansion *)
	  | ext of predicate; (* predicate extension *)
		
	    
	(* type of object *) 
	datatype obj =
	    initial | terminal 
	  | sing of  assertion       (* single object *)
	  | max of (assertion list)  (* maximum object *)
	  | min of (assertion list); (* minimum object *)
  
	(* arrow *)
	type 'obj arrow = 'obj*'obj;
       
	(* abstract category *)
        type 'obj category = ('obj list*('obj)arrow list);

	(* our category *)    
	type catg = obj category;
	    
        (* pre-category *)
	type precat = catg;

        (* post-category linked with the respective 
	 antecedent from the pre-category *)
	type poscat = obj*catg;

	(* post-categories *)
        type poscats = poscat list;

	(* name as identifier *)
	type name = string;
	    
	(* method *)
        type method = name*precat*poscats;

	(* set of constants which defines sort *) 
	type sort =
	    id list;
	    
	(* methods *)
	type methods = 
	    method list;
	    
	(* component *)
        type component = name*sort*methods;

    end;
    