(***********************************************************************************
 *
 *  Application SRTFS (Software Reuse Through Formal Specifications) 
 *
 *  File: data_ctg.sml
 *
 *  Description: data structures used in the representations of
 *               algebraic specifications
 * 
 ***********************************************************************************)

structure Spec =
    struct 

	(* identifier *)
	type id = string;
        
	(* constant *)
	type tconst = id;

	(* type of variable *)
	datatype tvar =    
	    st of id |  (* state variable *)
            in_ of id | 
            out of id;  
       
	(* type of term *)
	datatype term =
	    const of tconst | (* constant *)
	    normal of id | (* variable quantified *)
	    var of tvar; (* variable defined *)
	    
	(* type of predicate *)
        datatype predicate =
	    pred of id   (* normal predicate *)
	  | comp of tvar; (* equality predicate *)

	(* type of pseudo-proposition *)
       	datatype pproposition =
	    true | false |
            prop of predicate*term | (* term applied to predicate *)
	    all of predicate*id |    (* universal quantifier
				      applied to a predicate *)
	    one of predicate*id;    (* existential quantifier
				      applied to a predicate *)
	    
	(* part of a clause *)
	datatype sub_clause =
	    conj of pproposition list| (* antecedent *)
	    disj of pproposition list; (* consequent *)

	(* clause *) 
	datatype clause =
	    prop_impl of sub_clause*sub_clause

	(* clausal affirmation *)
	type clauses = 
	    clause list;

	(* post-condition *)
	type poscondition = 
	    sub_clause*clauses;

	(* name as identifier *)
	type name = string; 

	(* set of constants which defines sort *) 
	type sort =
	    tconst list;

	(* state variables *)
	type variables =
	    tvar list;

	(* invariant *)
	type invariant =
	    clauses;

	(* interface variables *)
	type interface =
	    tvar list;

	(* pre-condition *)
        type requires =
            sub_clause list;

	(* post-conditions *)
	type ensures =
	    poscondition list;

	(* method *)
	type method =
	    name*interface*requires*ensures;

	(* methods *)
	type methods =
	    method list;

	(* component *)
        type component =
	    name*sort*variables*invariant*methods;

    end;



