(***********************************************************************************
 *
 *  Application SRTFS (Software Reuse Through Formal Specifications) 
 *
 *  File: errors.sml
 *
 *  Description: functions for treating errors 
 * 
 ***********************************************************************************)

(* functions that help the debug *)
fun debug_ s = print s;
fun debug s =  ();
    
structure Err =
    struct
	
	fun write messg =
	    TextIO.output(TextIO.stdErr, messg);
	    
	(* type of possible errors *)
	exception non_composable_pair;
	exception invalid_prod;
	exception write_file;
	exception read_file;
	exception data_file;
	exception dir;
        exception bug;
	exception bad_args;

	exception var_st;
	exception var_def;
	exception var_use;
	exception var_itf;

	(* errors messages *)
	fun error_out typ par =
	    (write (case typ of
			non_composable_pair => "Non composable pair"
		      | invalid_prod => "Invalid product"
		      | write_file => "Can't write in the file: "
		      | read_file => "Can't read the file: "
		      | dir => "Can't open the directory: "
		      | data_file => "Bad data in the file: "
		      | bad_args => "Bad args: SRTFS <lib_dir> <name_spec>"
		      | bug => "A internal bug was detected : "
		      | var_st => "Only defined variables at left of ="
		      | var_def => "Only constants are permitted"
		      | var_use => "Isn't used the variable: "
		      | var_itf => "Interface variable not defined"
		      | _ => "Unknown error"); 
	     write (par^"\n");
	     raise typ)
		 
	(* treat semantic errors *) 
	fun error typ par = 
	    ( write "Error, ";
	     error_out typ par)
	     
	(* treat syntax specification errors *)
	fun spc_error l typ par = 
	    (write ("Specification Error, line "^(Int.toString l)^", ");
	     error_out typ par)

    end





