(***********************************************************************************
 *
 *  Application SRTFS (Software Reuse Through Formal Specifications) 
 *
 *  File: index.sml
 *
 *  Description: management of the specifications repository  
 * 
 ***********************************************************************************)

structure Index =
    struct

	exception BadFile

	val ctg_index = "category.idx";

	(* returns the list of components present in the repository *) 
	fun read () =
	    let
		val is = TextIO.openIn (ctg_index)
		    handle IO  => Err.error Err.read_file (ctg_index)
		val ret = ref [] : (string*int) list ref;
	    in
		while not (TextIO.endOfStream is) do
		    let val line = TextIO.inputLine is
			(* split the name from the sort length*) 
			val aux = String.tokens 
			    (fn #" " => true | _ => false) line;
			val _ = if (List.length aux) <> 2 
				    then Err.error Err.data_file (ctg_index)
				else ();
			val pair = 
			    case Int.fromString (hd(tl aux)) of
				SOME x => (hd aux, x)
			      | _ => Err.error Err.data_file (ctg_index)
		    in
			(* join one component to the list *)
			ret := pair::(!ret)
		    end;
		    TextIO.closeIn is;
		    (!ret)
	    end;

	(* insert in the repository the name and the sort length
	 of a component *)
	fun insert (pair as (name,n)) =
	    let 
		val actual = read ()
		(* verifies if already exists in the repository *)
		val aux = List.filter (fn x => not(name = (#1 x))) actual
		val is = ((if (List.length aux) = (List.length actual)
			       then TextIO.openAppend (ctg_index)
			   else TextIO.openOut (ctg_index))
			  handle IO  => Err.error Err.write_file (ctg_index))
		fun write (name,n) =
		    TextIO.output (is, name^" "^(Int.toString n)^"\n")
	    in
		(* new insert *) 
		if (List.length aux) = (List.length actual)
		    then write pair
		(* update *)
		else (List.app
		      (fn pair => write pair)
		      (pair::aux));
		    TextIO.closeOut is
	    end;

    end






