(***********************************************************************************
 *
 *  Application SRTFS (Software Reuse Through Formal Specifications) 
 *
 *  File: match.sml
 *
 *  Description: management of the operations made by the application
 * 
 ***********************************************************************************)

structure Match =
    struct

	structure Iso = IsoMatch;
	structure Comp = CompMatch;

	(* apply one type of match between the user
	 categorical specification and all the library
	 categorical specifications *)
 	fun match type_match user_ctg lib_ctgs =
	    List.app 
	    (fn (lib_name,_) => 
	     (type_match user_ctg (ParserCtg.parse lib_name)
	      handle Err.read_file => ()))
	    lib_ctgs

	(* In: dir = repository's directory 
	 *     name = name of the user specification
	 * Out: nothing 
	 * Description: makes all the operations with the user specification. *)
	fun main dir name =
	    let
		val _ = (OS.FileSys.chDir dir
			 handle _ => Err.error Err.dir dir) 
		(* translates the user algebraic specification *) 
		val user_ctg = ParserSpc.parse name
		(* write the user categorical specification *)
		val _ = Output.component name user_ctg
		val _ = print "The categorical representation was created.\n" 
		val sort_size = (List.length(#2 user_ctg))
		(* filter the library specifications by their sort size *)
		val lib_ctgs = (List.filter 
				(fn (lib_name,lib_n) => 
				 not(name=lib_name) andalso (sort_size<=lib_n))
				(Index.read ()))
	    in
		print "Isomorphic matching\n";
		match Iso.match_comp user_ctg lib_ctgs;
		print "\nComposite matching\n";
		match Comp.match_comp user_ctg lib_ctgs;
		OS.Process.success
	    end

        (* treats a possible error and the arguments given *)
	fun start (name :string, args) =
	    let
		val old_dir = OS.FileSys.getDir ()
                val res = 
		    if (List.length args) = 2 then  
			(main (List.hd args) (List.hd (List.tl args))
			 handle _ => OS.Process.failure)
			else (Err.error Err.bad_args "";
			      OS.Process.failure)
	    in
		
		if res = OS.Process.success then print "The End\n"
		else print "The application was aborted\n";
		    OS.FileSys.chDir old_dir;
		    res
	    end;

    end










