(***********************************************************************************
 *
 *  Application SRTFS (Software Reuse Through Formal Specifications) 
 *
 *  File: out_ctg.sml
 *
 *  Description: writes one categorical specification in a file
 * 
 ***********************************************************************************)

structure Output =
    struct

	structure Catg=Catg;
	
	(* prints the elements of a list separated by one symbol *) 
	fun print_list print_comma print_elem list  =
	    let 
		val rec plist =
		    fn [x] => print_elem x
		     | x::rest => (print_elem x;
				   print_comma ();
				   plist rest)
		     | _ => ()
	    in
		plist list
	    end

	(* prints one variable *)
	fun var v =
	    case v of
	        Catg.in_ (id) => "?"^id
	      | Catg.out (id) => "!"^id;

	(* prints one term *)
	fun term t =
	    case t of 
		Spec.const(id) => id 
	      | _ => Err.error Err.bug "Out.term"

	
	(* prints one predicate *)
	fun pred p =
	    case p of
		Catg.pred (id) => id
	      | Catg.comp (v) => (var v)^"="
		    
		    
	(* prints one object, out defines where is printed *)
	fun object_ out obj = 
	    let		
		(* prints one assertion *)
		fun assertion p = 
		    case p of 
			Catg.prop(p,t) => out ((pred p)^"("^t^")")
		      | Catg.ext(p) => out ("+"^(pred p))
		      | Catg.exp(p) => out ("*"^(pred p))
			
	    in
		case obj of
		    Catg.terminal => out "1"
		  | Catg.initial => out "0"
		  | Catg.sing (x) => (out("#{");
				      assertion x;
				      out("}") )
		  | Catg.min (x) => ( out("&{");
				     print_list (fn()=>out(",")) assertion x;
				     out ("}")
				     )
		  | Catg.max (x) => ( out("|{");
				     print_list (fn()=>out(",")) assertion x;
				     out ("}")
				     )
	    end;
 
	(* prints one object and changes line, out defines where is printed *)
	fun object out obj = 
	    ( object_ out obj;
	     out ("\n") )
	
	(* prints one arrow and changes line, out defines where is printed *)
	fun arrow out (origin,destine) =
	    ( 
	     object_ out origin;
	     out(" => ");
		 object_ out destine; 
		 out ("\n")
		 )

	(* prints one category, out defines where is printed *)
	fun category out (objs,arrs) =
	    ( out ("\tObjects: \n");
	     List.app (object out) objs;
	     out ("\tArrows: \n");
	     List.app (arrow out) arrs )


	(* prints one post-category and its antecedent, 
	 out defines where is printed *)
	fun pos_category out (obj, catg) =
	    (
	     out ("\n\tPostCategory:\n");
	     object_ out obj;
	     out (" -> \n");
	     category out catg
	     )
	
	(* prints one method, out defines where is printed *)
	fun method out (name, precat, poscat) =
	    ( out ("\nMethod: "^name^"\n");
	     out ("\n\tPreCategory:\n");
	     category out precat;
	     List.app (pos_category out) poscat
	     )

	(* prints the set sort, out defines where is printed *)
	fun sort out (X) =
	    ( out ("\n\nSort: {");
	     print_list (fn()=>out(",")) (fn x=>out(x)) X;
	     out ("}\n")
	     )

	(* prints one component, out defines where is printed *)
	fun component name_fich (name, X, metds) =
	    let 
		val f = (TextIO.openOut (name_fich^".ctg")
			 handle IO  => Err.error Err.write_file (name_fich^".ctg"))
		fun out s = TextIO.output (f, s);
	    in
		out ("Categorical Representation\n");
		out ("Component "^name^"\n");
		sort out X;
		List.app  (method out) metds;
		TextIO.closeOut f;
		Index.insert (name_fich, (List.length X))
	    end;	    
    
    end;








