(***********************************************************************************
 *
 *  Application SRTFS (Software Reuse Through Formal Specifications) 
 *
 *  File: utils.sml
 *
 *  Description: utilities functions 
 * 
 ***********************************************************************************)

structure Utils =
    struct
	
	(* In: p = a two arity predicate to compare elements
	 *     and two lists 
	 * Out: true or false
	 * Description: tells if all elements of the first list belong 
	 *              to the second one *) 
	fun setcont_ p (h::t, l) = 
	    List.exists (fn x => (p x h)) l andalso
	    setcont_ p (t,l)
	  |setcont_ _ ([],_) = true;
    
	(* In: p = a two arity predicate to compare elements
	 *     and two lists 
	 * Out: true or false
	 * Description: tells if the lists have the same elements *)
	fun seteq_ p (l1,l2) = setcont_ p (l1,l2) andalso setcont_ p (l2,l1);
	    
	(* In: two lists 
	 * Out: true or false
	 * Description: tells if all elements of the first list belong 
	 *              to the second one *) 
	fun setcont (l1,l2) = setcont_ (fn x => fn y => x=y) (l1, l2) 
	
        (* In: two lists 
	 * Out: true or false
	 * Description: tells if the lists have the same elements *)
	fun seteq (l1,l2) = setcont  (l1,l2) andalso setcont  (l2,l1);
	
	(* In: f = a two arity function 
	 *     i = the initial value
	 *     and a list 
	 * Out\Description: the result of apply the function f
	 *                  to all the members of the list using 
         *                  the previous results too *) 
	fun nestlist f i [] = i
	  | nestlist f i (x::r) =
	    nestlist f (f x i) r;
	    

	(* In: p = a two arity predicate to compare elements 
	 *     and two lists 
	 * Out: a list
         * Description: gives the common elements of the two lists *)  
	fun inter p l1 l2 = 
	    nestlist (fn x => fn l => 
		      if (List.exists (fn y => (p x y)) l2) then
			  x::l
		      else l
			  ) [] l1

	(* In: x = a element 
	 *     list = a list
	 * Out: a list
         * Description: insert x in the list without duplication *)  
	fun insert x list =
	    if List.exists (fn elem => x=elem) list then
		list
	    else 
		x::list;

	(* In: two lists 
	 * Out: a list
         * Description: append the two lists without duplication *)  
	fun append l1 l2 =
	    nestlist insert l1 l2;

	(* In: p = a two arity predicate
	 * Out: a list
         * Description: gives the rest of the list when it finds one
	 *              element that satisfies the predicate *)  
	fun filter_rest p = 
	    fn [] => []
	     | h::t => if (p h) then t 
		       else h::(filter_rest p t)
	
        (* gives the value of a SOME *)   
	val get_some =
	    fn NONE => Err.error Err.bug "Utils.get_some"
	     | SOME(x) => x
	    
    end;





    